import os, sys, arcpy, time
from datetime import datetime

dt = datetime.now()
CurrentTime = str(dt.hour) + ':' + str(dt.minute) + ':' +str(dt.second) + ' ' + \
              str(dt.year)+'.'+str(dt.month)+'.' + str(dt.day)
CurrentDate = str(dt.day) + '/' + str(dt.month) + '/' + str(dt.year)
ModifiedDate = 'Ver.24JAN2011 \n(Updates: using native ArcGIS 10 ArcPy sitepackage'

def Printboth(msg):
    """<string> -> print to both command line and arcmap geoprocessing window"""
    print msg
    arcpy.AddMessage(msg)

def GetUniqueValuesFromShapefile(inputShp, inputField):
    """<string>, <string> -> pythonList
    can be both feature class or feature layer"""
    layer = "featurelayer"
    arcpy.MakeFeatureLayer_management(inputShp, layer)

    rows = arcpy.SearchCursor(layer)
    row = rows.next()

    pyList = []
    while row:
        if row.getValue(inputField) not in pyList:
            pyList.append(row.getValue(inputField))
        row = rows.next()
    del row, rows
    arcpy.Delete_management(layer)
    return pyList

def GetUniqueValuesFromFeatureLayer_mk2(inputFc, inputField):
    """<string>, <string> -> pythonList
    can be both feature class or feature layer"""
    pySet = set()
    with arcpy.da.SearchCursor(inputFc, inputField) as cursor:
        for row in cursor:
            pySet.add(row[0])

    return list(pySet)

def GetUniqueValuesFromTable(inputTable, inputField):
    """<string>, <string> -> pythonList
    can be both feature class or feature layer"""
    #layer = "featurelayer"
    #arcpy.MakeFeatureLayer_management(inputShp, layer)

    rows = arcpy.SearchCursor(inputTable)
    row = rows.next()

    pyList = []
    while row:
        if row.getValue(inputField) not in pyList:
            pyList.append(row.getValue(inputField))
        row = rows.next()
    del row, rows
    #arcpy.Delete_management(layer)
    return pyList



def CalcuateTotalArea(inputshp, field = '#', spatialref='#'):
    """<shp/lyr>, {summarize field}, {spatialref}--> python dictionary
    Input shapefile MUST have coordinate systems!!
    this function is used to calcuate total areas of the input shapefile or layer
    , optionally in a user defined spatial reference system (projection on the fly)
    and/or aggregated by a field specified by user
    function added 101124"""
    #optional spatial reference
    dict = {}
    if spatialref == '#':
        spatialreference = ''
    else:
        spatialreference = spatialref

    #optional field clause, if field is not specified
    if field == '#' or field == '':
        rows = arcpy.SearchCursor(inputshp, '', spatialreference)
        row = rows.next()
        # reset area value
        area = 0
        while row:
            area += row.shape.area
            row = rows.next()
        dict['totalarea'] = area
        return dict     

    # if field is specified    
    else:
        uniquevalues = GetUniqueValuesFromShapefile(inputshp, field)

    # if given field has unique values        
    if uniquevalues:
        for eachvalue in uniquevalues:
            # need to separate string with numbers, also uniquecode
            if type(eachvalue) == type('a') or type(eachvalue) == type(u'a'):
                whereclause = "\"" + field + "\" = '" + eachvalue + "'"
            else:
                whereclause = "\"" + field + "\" = " + str(eachvalue)
                
            rows = arcpy.SearchCursor(inputshp, whereclause, spatialreference)
            row = rows.next()
            # reset area value
            area = 0
            while row:
                area += row.shape.area
                row = rows.next()
                
            dict[eachvalue] = area
            del row, rows
    return dict

def SingeQuoteToDoubleSQL(strName):
    """<string> -> <string>
    converts to a valid name that is stripped and change ' to '' for SQL statement"""
    if '\'' in strName:
        validstrName = strName.replace('\'', '\'\'')
    else:
        validstrName = strName
        
    return validstrName

def Log_output(msg, textfile = 'result.txt'):
    """<string>, <string>"""
    """ <string>, {text file name}, will be saved under arcpy.env.workplace\ default: result.txt}
    This function is called to record the current status, error message, time etc"""
    fi = open(arcpy.env.workspace + os.sep + textfile, 'a')
    fi.write(msg)
    fi.close()

def CleanSpeciesName(strName):
    """ This function cleans species name by rconverting hyphens and spaces to underscore """
    s = strName.replace(" ","_")
    validstrName = s.replace("-","_")

    return validstrName

def AddFieldWithValue (Table, Fieldname, Fieldtype, Value):   
    """ This function add a field if it does not already exist and also calculates the value
    When calculating a string value enclose string in single quotes """
    # Check field exists
    if  len(arcpy.ListFields(Table, Fieldname)) <> 0:
        # Set value
        arcpy.CalculateField_management(Table, Fieldname, Value, "PYTHON")             
    else:
        # Add field and set value
        arcpy.AddField_management(Table, Fieldname, Fieldtype)
        arcpy.CalculateField_management(Table, Fieldname, Value, "PYTHON")

def AddField (Table, Fieldname, Fieldtype):
    """ This function adds a field if it does not already exist """
    # Check field exists
    if  len(arcpy.ListFields(Table, Fieldname)) == 0:
        # Add field
        arcpy.AddField_management(Table, Fieldname, Fieldtype)

def CheckFieldExists (Table, Fieldname):
    """ This function checks if a field exists """
    if arcpy.ListFields(Table,Fieldname):
        fieldcheck = True
    else:
        fieldcheck = False
        
    return fieldcheck
            


def JoinTableToBaseLayer(BaseLayer, BaseField, TableView, TableField, JoinType = "KEEP_COMMON"):
    """ Simple join function """
    arcpy.AddJoin_management(BaseLayer, BaseField, TableView, TableField, JoinType)  
    
    
if __name__ != "__main__":
    print "Yichuan10's header imported at %s, %s"%(CurrentTime,ModifiedDate)
else:
    pass
